import glob
import math
import os
import tempfile
import threading
import time
import atexit
import shutil
import pygameextra as pe
import pygameextra_cool_buttons
from pygameextra_cool_buttons.color import *
from functools import wraps, lru_cache
from typing import Type, Generator, Tuple, Union
from PIL import Image

pe.settings.cb_default_edge_rounding = 4

pe.init()

SCRIPT_DIR = os.path.dirname(__file__)
RECORDING_DIRECTORY = os.path.join(SCRIPT_DIR, 'recording')
os.makedirs(RECORDING_DIRECTORY, exist_ok=True)
RECORDING_IN_PROGRESS = False


class ButtonRecorderMixin(pe.Button):
    RECORDING_TEXT: str = 'Hello, PGE!'
    RECORDING_PADDING: Tuple[int, int] = (100, 100)
    RECORDING_SURFACE_AREA: Tuple[int, int] = (300, 150)
    RECORDING_SURFACE_BACKGROUND: Tuple[int, int, int] = pe.colors.darkgray
    RECORDING_FAKE_OUTLINE: int = 15
    RECORDING_FAKE_INLINE: int = 5
    RECORDING_FRAMES = 40
    RECORDING_TIME_IN = 4
    RECORDING_TIME_HOLD = 4
    RECORDING_CLICK_HOLD = .1
    RECORDING_TIME_OUT = 4
    RECORDING_TIME = RECORDING_TIME_IN + RECORDING_TIME_HOLD + RECORDING_TIME_OUT

    button_name: str
    recording: bool
    recording_state: int
    recording_start: float
    recording_capture_index: int
    recording_temp_dir: str
    _hovered: bool
    _area: Tuple[int, int, int, int]
    _surface: pe.Surface = None
    spoofed_mouse_position: Tuple[int, int] = None
    original_area: Tuple[int, int, int, int]

    # state 0: mouse going in
    # state 1: mouse in
    # state 2: mouse going out

    @property
    def elapsed(self):
        return time.time() - self.recording_start

    @property
    @lru_cache()
    def recording_area(self):
        return (
            *tuple(padding // 2 for padding in self.RECORDING_PADDING),
            *tuple(
                size - padding for size, padding in
                zip(self.RECORDING_SURFACE_AREA, self.RECORDING_PADDING)
            )
        )

    def handle_recording(self):
        global RECORDING_IN_PROGRESS
        if self.elapsed < self.RECORDING_TIME_IN:
            self.recording_state = 0
        elif self.elapsed < self.RECORDING_TIME_IN + self.RECORDING_TIME_HOLD:
            self.recording_state = 1
        elif self.elapsed < self.RECORDING_TIME:
            self.recording_state = 2
        elif self.elapsed >= self.RECORDING_TIME:
            self.recording = False
            self.recording_state = -1
            threading.Thread(target=self.save_gif, daemon=True).start()
            RECORDING_IN_PROGRESS = False

    def save_gif(self):
        frames = [
            Image.open(image)
            for image in sorted(
                glob.glob(
                    f"{self.recording_temp_dir}/*.png"),
                key=lambda x: int(os.path.basename(x).split('.')[0])
            )
        ]
        frame_one = frames[0]
        frame_one.save(os.path.join(RECORDING_DIRECTORY, f"{self.button_name}.gif"), format="GIF",
                       append_images=frames,
                       save_all=True, fps=self.RECORDING_FRAMES / 1.5,
                       loop=0)

    @property
    def area(self):
        return self._area if not self.recording else self.recording_area

    def _render(self, *args, **kwargs):
        super().render(*args, **kwargs)

    def _logic(self, *args, **kwargs):
        super().logic(*args, **kwargs)

    @property
    def surface(self):
        if not self._surface:
            self._surface = pe.Surface(self.RECORDING_SURFACE_AREA)
        pe.fill.full(self.RECORDING_SURFACE_BACKGROUND, self._surface)
        return self._surface

    def get_fake_mouse_position(self):
        t = 0
        if self.recording_state == 0:
            t = min(1, (self.elapsed / self.RECORDING_TIME_IN))
        elif self.recording_state == 1:
            t = 1
        else:
            t = 1 - min(1, (self.elapsed - self.RECORDING_TIME_IN - self.RECORDING_TIME_HOLD) / self.RECORDING_TIME_OUT)
        return pe.math.lerp(
            tuple(size - (padding // 4) for size, padding in
                  zip(self.RECORDING_SURFACE_AREA, self.RECORDING_PADDING)),
            tuple(size // 2 for size in self.RECORDING_SURFACE_AREA),
            t
        )

    def logic(self, *args, **kwargs):
        if not self.recording:
            self._logic(*args, **kwargs)
        else:
            fake_mouse_position = self.get_fake_mouse_position()

            @pe.mouse.offset_wrap(
                tuple(
                    -position + fake_position for position, fake_position in
                    zip(pe.mouse.pos(), fake_mouse_position)))
            def wrapped():
                self._logic(*args, **kwargs)
                self.spoofed_mouse_position = pe.mouse.pos()

            if self.spoof_mouse_click:
                pe.settings.spoof_mouse_clicked = (True, False, False)
            wrapped()
            pe.settings.spoof_mouse_clicked = None

    @property
    def spoof_mouse_click(self):
        return self.recording_state == 1 and self.elapsed < self.RECORDING_TIME_IN + self.RECORDING_CLICK_HOLD

    def render(self, *args, **kwargs):
        pe.draw.rect((*pe.colors.black, 10), self.original_area, 2, edge_rounding=pe.settings.cb_default_edge_rounding)
        if not self.recording:
            self._render(*args, **kwargs)
        else:
            frame = int(self.RECORDING_FRAMES * (self.elapsed / self.RECORDING_TIME))

            @pe.display.context_wrap(self.surface)
            def wrapped():

                self._render(*args, **kwargs)

                if self.recording_capture_index < frame:
                    if self.spoof_mouse_click == 1:
                        pe.draw.circle(
                            (*pe.colors.black, 50),
                            self.spoofed_mouse_position or self.get_fake_mouse_position(),
                            self.RECORDING_FAKE_OUTLINE, 3
                        )
                    pe.draw.circle(
                        (*pe.colors.black, 50),
                        self.spoofed_mouse_position or self.get_fake_mouse_position(),
                        self.RECORDING_FAKE_INLINE, 0
                    )

                    self.recording_capture_index = frame
                    pe.pygame.image.save(self._surface.surface,
                                         os.path.join('recording',
                                                      os.path.join(self.recording_temp_dir, f'{frame}.png')))

            wrapped()

    @area.setter
    def area(self, value):
        self._area = value

    @property
    def hovered(self):
        return self._hovered

    @hovered.setter
    def hovered(self, value):
        self._hovered = value

    @staticmethod
    @lru_cache
    def _get_text(text, size):
        return pe.Text(text, font_size=size)

    @property
    def text(self):
        return self._get_text(
            self.button_name if not self.recording else self.RECORDING_TEXT,
            self.area[3] // 2
        )

    @text.setter
    def text(self, value):
        pass


def class_recordable_wrapper(cls: Type[pe.Button]):
    class RecordableButton(ButtonRecorderMixin, cls):
        # For anyone copying this, it only works within contexts
        def __init__(self, area, *args, **kwargs):
            super().__init__(area, *args, **kwargs)
            self.button_name = None
            self.recording_temp_dir = None
            self.recording = False
            self.recording_state = -1
            self.recording_start = 0
            self.recording_capture_index = -1
            self.original_area = area

        def logic(self, *args, **kwargs):
            global RECORDING_IN_PROGRESS
            super().logic(*args, **kwargs)
            if self.hovered and not self.recording and pe.settings.game_context.trigger_recording and not RECORDING_IN_PROGRESS:
                pe.settings.game_context.trigger_recording = False
                RECORDING_IN_PROGRESS = True
                self.recording = True
                self._hovered = False
                self.recording_state = 0
                self.recording_capture_index = -1
                self.recording_start = time.time()
            elif self.recording:
                self.handle_recording()

    return RecordableButton


def button_naming_wrapper(func):
    @wraps(func)
    def wrapper(*args, button_name=None, **kwargs):
        func(*args, **kwargs)

        # Retain buttons instead of generating new ones
        for previous_button in pe.settings.game_context.previous_buttons:
            if previous_button.button_name == button_name:
                del pe.settings.game_context.buttons[-1]
                pe.settings.game_context.buttons.append(previous_button)
                break
        if not pe.settings.game_context.buttons[-1].button_name:
            pe.settings.game_context.buttons[-1].button_name = button_name
            pe.settings.game_context.buttons[-1].recording_temp_dir = tempfile.mkdtemp()
            atexit.register(shutil.rmtree, pe.settings.game_context.buttons[-1].recording_temp_dir)
        pe.button.check_hover(pe.settings.game_context.buttons[-1])

    return wrapper


def button_check_hover_wrapper(func):
    @wraps(func)
    def wrapper(button, *args, **kwargs):
        # Don't display buttons that are not named
        if not button.button_name:
            return
        func(button, *args, **kwargs)

    return wrapper


# Wrap pygameextra button components to allow for button recording
if not hasattr(pe.settings, 'wrapped'):
    pe.button.check_hover = button_check_hover_wrapper(pe.button.check_hover)

    pe.button.Button = class_recordable_wrapper(pe.button.Button)
    pe.button.RectButton = class_recordable_wrapper(pe.button.RectButton)
    pe.button.ImageButton = class_recordable_wrapper(pe.button.ImageButton)

    pe.button.action = button_naming_wrapper(pe.button.action)
    pe.button.rect = button_naming_wrapper(pe.button.rect)
    pe.button.image = button_naming_wrapper(pe.button.image)
    setattr(pe.settings, 'wrapped', True)


class Context(pe.GameContext):
    AREA = (800, 500)
    BACKGROUND = pe.colors.gray
    TITLE = 'Cool Buttons Tester'
    BUTTON_SIZE = (120, 30)
    BUTTON_PADDING = 10
    positions: Generator[Tuple[int, int], None, None]
    COLOR_A = pe.colors.purple
    COLOR_B = pe.colors.darkaqua
    COLOR_B_PULSING = PulsingColor(GradientColor(
        PartialGradientColor(Color(pe.colors.verydarkaqua), 0),
        PartialGradientColor(Color(pe.colors.darkcyan), .5),
        PartialGradientColor(Color(pe.colors.aquamarine), 1)
    ), .1, 0, 1)
    COLOR_A_PULSING = PulsingColor(GradientColor(
        PartialGradientColor(Color(pe.colors.red), 0),
        PartialGradientColor(Color(pe.colors.orange), .5),
        PartialGradientColor(Color(pe.colors.red), 1)
    ), 2, .5, 2)
    IMAGE_A: pe.Image
    IMAGE_B: pe.Image
    FPS_LOGGER = True

    def __init__(self):
        super().__init__()
        self.trigger_recording = False
        self.IMAGE_A = pe.Image(os.path.join(SCRIPT_DIR, 'IMAGE_A.png'), self.BUTTON_SIZE)
        self.IMAGE_B = pe.Image(os.path.join(SCRIPT_DIR, 'IMAGE_B.png'), self.BUTTON_SIZE)

    def handle_event(self, _):
        super().handle_event(_)
        if pe.event.key_DOWN(pe.K_r):
            self.trigger_recording = True

    @classmethod
    def position_generator(cls):
        x_and_padding = cls.BUTTON_SIZE[0] + cls.BUTTON_PADDING
        y_and_padding = cls.BUTTON_SIZE[1] + cls.BUTTON_PADDING
        for x in range(cls.BUTTON_PADDING, cls.AREA[0] - x_and_padding, cls.BUTTON_SIZE[0] + cls.BUTTON_PADDING):
            for y in range(cls.BUTTON_PADDING, cls.AREA[1] - y_and_padding, cls.BUTTON_SIZE[1] + cls.BUTTON_PADDING):
                yield x, y

    @property
    def button_area(self):
        return *next(self.positions), *self.BUTTON_SIZE

    def loop(self):
        pe.button.action(self.button_area, button_name='pe.button.action')
        pe.button.rect(self.button_area, self.COLOR_A, self.COLOR_B, button_name='pe.button.rect')
        pe.button.image(self.button_area, self.IMAGE_A, self.IMAGE_B, button_name='pe.button.image')

        pe.button.rect(self.button_area, self.COLOR_A, self.COLOR_B, button_name='shadow', shadow=True)
        pe.button.rect(self.button_area, self.COLOR_A_PULSING, self.COLOR_B_PULSING, button_name='pulsing gradient')

    def pre_loop(self):
        super().pre_loop()
        self.positions = self.position_generator()


def run():
    context = Context()
    while True:
        context()


if __name__ == '__main__':
    run()



from typing import Union

from pygameextra import draw
from pygameextra.button import RectButton


class RectButtonExpansion(RectButton):
    @staticmethod
    def static_render(area: tuple, inactive_resource=None, active_resource=None, hovered: bool = False,
                      disabled: Union[bool, tuple] = None, shadow: bool = False,
                      shadow_color: tuple = (0, 0, 0, 50), shadow_offset: tuple = (2, 2),
                      edge_rounding: int = -1,
                      edge_rounding_topright: int = -1, edge_rounding_topleft: int = -1,
                      edge_rounding_bottomright: int = -1, edge_rounding_bottomleft: int = -1
                      ):
        color = active_resource if (hovered and not disabled) else (
            disabled if type(disabled) == tuple else inactive_resource)
        draw.rect(
            color, area,
            edge_rounding=edge_rounding,
            edge_rounding_topright=edge_rounding_topright,
            edge_rounding_topleft=edge_rounding_topleft,
            edge_rounding_bottomright=edge_rounding_bottomright,
            edge_rounding_bottomleft=edge_rounding_bottomleft
        )


button_expansion_map = {
    RectButton: RectButtonExpansion
}



from abc import ABC, abstractmethod
from typing import List, Tuple

from pygameextra import settings


class UniqueColor(ABC):
    class Info(ABC):
        pass

    @abstractmethod
    def get_color(self, info: Info) -> tuple:
        raise NotImplementedError("Method get_next_color not implemented")


class ColorWithPercentageMixin:
    class Info:
        def __init__(self):
            self.percentage = 0


class ColorWithPercentage(UniqueColor, ColorWithPercentageMixin, ABC):
    pass


class Color(UniqueColor):

    def __init__(self, color: tuple):
        self.color = color

    def get_color(self, info) -> tuple:
        return self.color


class PartialGradientColor:
    def __init__(self, color: UniqueColor, percentage: float):
        self.color = color
        self.percentage = percentage


class GradientColor(ColorWithPercentage):
    class Info(ColorWithPercentageMixin.Info):
        def __init__(self):
            super().__init__()
            self.sub_infos = None


    def __init__(self, *colors: PartialGradientColor):
        self.colors: Tuple[PartialGradientColor, ...] = tuple(sorted(colors, key=lambda x: x.percentage))

    @staticmethod
    def _interpolate(color1: tuple, color2: tuple, percentage: float) -> tuple:
        return tuple(int(color1[i] * (1 - percentage) + color2[i] * percentage) for i in range(3))

    def get_color_at(self, percentage: float, info: Info) -> tuple:
        if percentage < 0 or percentage > 1:
            raise ValueError("Percentage should be between 0 and 1")
        for i in range(len(self.colors) - 1):
            if self.colors[i].percentage <= percentage <= self.colors[i + 1].percentage:
                return self._interpolate(self.colors[i].color.get_color(info.sub_infos[i]), self.colors[i + 1].color.get_color(info.sub_infos[i + 1]),
                                         (percentage - self.colors[i].percentage) / (
                                                 self.colors[i + 1].percentage - self.colors[i].percentage))

    def get_color(self, info: Info) -> tuple:
        if info.sub_infos is None:
            info.sub_infos = [color.color.Info() for color in self.colors]
        return self.get_color_at(info.percentage, info)


class PulsingColor(ColorWithPercentageMixin, UniqueColor):
    class Info(ColorWithPercentageMixin.Info):
        def __init__(self):
            super().__init__()
            self.pulse_state = 0
            self.time = 0

    def __init__(self, color: ColorWithPercentage, pulse_in: float, pulse_hold: float, pulse_out: float):
        self.color = color

        class CombinedInfo(color.Info, self.Info):
            pass
        self.Info = CombinedInfo
        self.pulse_in = pulse_in
        self.pulse_hold = pulse_hold
        self.pulse_out = pulse_out

    def get_color(self, info: Info) -> tuple:
        try:
            info.time += settings.game_context.delta_time
        except AttributeError:
            raise AttributeError("PulsingColor can only be used in a game context")

        if info.pulse_state == 0:
            if info.time >= self.pulse_in:
                info.time = 0
                info.percentage = 1
                info.pulse_state = 1
            else:
                info.percentage = info.time / self.pulse_in
        elif info.pulse_state == 1:
            if info.time >= self.pulse_hold:
                info.time = 0
                info.pulse_state = 2
        elif info.pulse_state == 2:
            if info.time >= self.pulse_out:
                info.time = 0
                info.percentage = 0
                info.pulse_state = 0
            else:
                info.percentage = 1 - info.time / self.pulse_out
        return self.color.get_color(info)



import logging
from functools import lru_cache, wraps
from typing import Type, Union

import pygameextra
import pygameextra.button as buttons
import pygameextra.settings as settings
from pygameextra import draw, mouse

from pygameextra_cool_buttons.color import UniqueColor
from pygameextra_cool_buttons.button_expansion import button_expansion_map

original_action = buttons.action
original_rect = buttons.rect
original_image = buttons.image


class WrappedButtonClass(buttons.Button):
    __base_button__ = buttons.Button

    def __init__(self, *args,
                 shadow: bool = False, shadow_color: tuple = (0, 0, 0, 50), shadow_offset: tuple = (2, 2),
                 edge_rounding: int = -1,
                 edge_rounding_topright: int = -1, edge_rounding_topleft: int = -1,
                 edge_rounding_bottomright: int = -1, edge_rounding_bottomleft: int = -1, **kwargs):
        self.infos = {}
        super().__init__(*args, **kwargs)
        self.edge_rounding = edge_rounding
        self.edge_rounding_topright = edge_rounding_topright
        self.edge_rounding_topleft = edge_rounding_topleft
        self.edge_rounding_bottomright = edge_rounding_bottomright
        self.edge_rounding_bottomleft = edge_rounding_bottomleft
        self.shadow = shadow
        self.shadow_color = shadow_color
        self.shadow_offset = shadow_offset

    def _color_translation(self, name: str, color: Union[bool, tuple, UniqueColor]):
        if not isinstance(color, UniqueColor):
            return color
        info = self.infos.get(name)
        if info is None:
            self.infos[name] = (info := color.Info())
        return color.get_color(info)

    @property
    def dynamic_area(self):
        return self.get_shadow_area(self.area, self.shadow_offset) if self.shadow and mouse.clicked()[0] else None

    @staticmethod
    def _edge_rounding_translation(name: str, edge_rounding: int):
        if edge_rounding == -1:
            return getattr(settings, f'cb_default_{name}')
        return edge_rounding

    @classmethod
    @lru_cache
    def get_shadow_area(cls, area: tuple, shadow_offset: tuple) -> tuple:
        rect = pygameextra.Rect(*area)
        rect.move_ip(*shadow_offset)
        return rect

    @classmethod
    def static_render(cls, area: tuple, inactive_resource=None, active_resource=None, hovered: bool = False,
                      disabled: Union[bool, tuple] = None, shadow: bool = False,
                      shadow_color: tuple = (0, 0, 0, 50), shadow_offset: tuple = (2, 2),
                      edge_rounding: int = -1,
                      edge_rounding_topright: int = -1, edge_rounding_topleft: int = -1,
                      edge_rounding_bottomright: int = -1, edge_rounding_bottomleft: int = -1,
                      dynamic_area: tuple = None
                      ):
        edge_rounding = cls._edge_rounding_translation('edge_rounding', edge_rounding)
        edge_rounding_topright = cls._edge_rounding_translation('edge_rounding_topright', edge_rounding_topright)
        edge_rounding_topleft = cls._edge_rounding_translation('edge_rounding_topleft', edge_rounding_topleft)
        edge_rounding_bottomright = cls._edge_rounding_translation('edge_rounding_bottomright', edge_rounding_bottomright)
        edge_rounding_bottomleft = cls._edge_rounding_translation('edge_rounding_bottomleft', edge_rounding_bottomleft)


        if shadow:
            draw.rect(
                shadow_color, cls.get_shadow_area(area, shadow_offset), 0,
                edge_rounding=edge_rounding,
                edge_rounding_topright=edge_rounding_topright,
                edge_rounding_topleft=edge_rounding_topleft,
                edge_rounding_bottomright=edge_rounding_bottomright,
                edge_rounding_bottomleft=edge_rounding_bottomleft
            )

        if cls.__base_button__ in button_expansion_map:
            button_expansion_map[cls.__base_button__].static_render(
                dynamic_area or area, inactive_resource, active_resource,
                hovered, disabled, shadow, shadow_color, shadow_offset, edge_rounding,
                edge_rounding_topright, edge_rounding_topleft,
                edge_rounding_bottomright, edge_rounding_bottomleft
            )
        else:
            super().static_render(dynamic_area or area, inactive_resource, active_resource, hovered, disabled)

    def render(self, area: tuple = None, inactive_resource=None, active_resource=None,
               text: pygameextra.Text = None, disabled: Union[bool, tuple, UniqueColor] = False, shadow: bool = False,
               shadow_color: tuple = (0, 0, 0, 50), shadow_offset: tuple = (2, 2),
               edge_rounding: int = -1,
               edge_rounding_topright: int = -1, edge_rounding_topleft: int = -1,
               edge_rounding_bottomright: int = -1, edge_rounding_bottomleft: int = -1
               ):
        inactive_resource = self._color_translation('inactive_resource', inactive_resource)
        self_inactive_resource = self._color_translation('self_inactive_resource', self.inactive_resource)
        active_resource = self._color_translation('active_resource', active_resource)
        self_active_resource = self._color_translation('self_active_resource', self.active_resource)
        disabled = self._color_translation('disabled', disabled)
        self_disabled = self._color_translation('self_disabled', self.disabled)
        if shadow:
            shadow_color = self._color_translation('shadow_color', shadow_color)
            self_shadow_color = self._color_translation('self_shadow_color', self.shadow_color)

        self.static_render(area or self.area, inactive_resource or self_inactive_resource,
                           active_resource or self_active_resource, self.hovered, disabled or self_disabled,
                           shadow or self.shadow, shadow_color or self_shadow_color,
                           shadow_offset or self.shadow_offset,
                           edge_rounding or self.edge_rounding, edge_rounding_topright or self.edge_rounding_topright,
                           edge_rounding_topleft or self.edge_rounding_topleft,
                           edge_rounding_bottomright or self.edge_rounding_bottomright,
                           edge_rounding_bottomleft or self.edge_rounding_bottomleft,
                           self.dynamic_area)
        self.static_render_text(area or self.area, text or self.text)


def button_class_wrapper(button_class: Type[buttons.Button]):
    class Wrapped(WrappedButtonClass, button_class):
        __base_button__ = button_class

        def __init__(self, *args, **kwargs):
            super().__init__(*args, **kwargs)

    return Wrapped


def button_function_wrapper(func):
    @wraps(func)
    def wrapped(*args, shadow: bool = False, shadow_color: tuple = (0, 0, 0, 50), shadow_offset: tuple = (2, 2),
                edge_rounding: int = -1,
                edge_rounding_topright: int = -1, edge_rounding_topleft: int = -1,
                edge_rounding_bottomright: int = -1, edge_rounding_bottomleft: int = -1, **kwargs):
        func(*args, **kwargs)
        if not settings.game_context and not hasattr(settings, 'cb_warn_function_wrapper'):
            logging.warning("Using the pygameextra button functions without a game context will not work properly")
            setattr(settings, 'cb_warn_function_wrapper', True)
            return
        button = settings.game_context.buttons[-1]
        button.shadow = shadow
        button.shadow_color = shadow_color
        button.shadow_offset = shadow_offset
        button.edge_rounding = edge_rounding
        button.edge_rounding_topright = edge_rounding_topright
        button.edge_rounding_topleft = edge_rounding_topleft
        button.edge_rounding_bottomright = edge_rounding_bottomright
        button.edge_rounding_bottomleft = edge_rounding_bottomleft
        setattr(button, 'cool_button', True)

    return wrapped


def expanded_button_check(button: WrappedButtonClass):
    if not hasattr(button, 'cool_button'):
        return
    elif not settings.game_context and not hasattr(settings, 'cb_warn_button_check_wrapper'):
        logging.warning("Using the pygameextra button functions without a game context will not work properly")
        setattr(settings, 'cb_warn_button_check_wrapper', True)
        return
    elif not settings.game_context:
        return
    elif not isinstance(button, WrappedButtonClass):
        return

    if len(settings.game_context.previous_buttons) >= (buttons_length := len(settings.game_context.buttons)):
        button.hovered = settings.game_context.previous_buttons[buttons_length - 1].hovered
        button.infos = settings.game_context.previous_buttons[buttons_length - 1].infos
        button.render()
        button.hovered = False
    else:
        button.hovered = False
        button.render()


# Wrap pygameextra buttons with extended functionality

if not hasattr(settings, 'cool_buttons'):
    buttons.check_hover = expanded_button_check

    buttons.Button = button_class_wrapper(buttons.Button)
    buttons.RectButton = button_class_wrapper(buttons.RectButton)
    buttons.ImageButton = button_class_wrapper(buttons.ImageButton)

    buttons.action = button_function_wrapper(original_action)
    buttons.rect = button_function_wrapper(original_rect)
    buttons.image = button_function_wrapper(original_image)

    setattr(settings, 'cool_buttons', True)
    setattr(settings, 'cb_default_edge_rounding', -1)
    setattr(settings, 'cb_default_edge_rounding_topright', -1)
    setattr(settings, 'cb_default_edge_rounding_topleft', -1)
    setattr(settings, 'cb_default_edge_rounding_bottomright', -1)
    setattr(settings, 'cb_default_edge_rounding_bottomleft', -1)



import pygameextra
import pygameextra_cool_buttons.buttons as cool_buttons
from pygameextra_cool_buttons.color import *


for attribute in dir(cool_buttons):
    if 'button' in attribute.lower():
        setattr(pygameextra, attribute, getattr(cool_buttons, attribute))


from setuptools import setup

version = "1.1"
short = 'Pygameextra cool buttons is a extension for pygameextra that adds cool buttons'
long = '''Pygameextra cool buttons is an extension for pygameextra,
that adds many button effects and customizations to spice up your app!

github: https://github.com/JustRedTTG/PGE-cool-buttons'''

# Setting up
setup(
    name="pygameextra_cool_buttons",
    version=version,
    author="Red",
    author_email="redtonehair@gmail.com",
    description=short,
    long_description_content_type="text/markdown",
    long_description=long,
    packages=['pygameextra_cool_buttons', 'pygameextra_cool_buttons_tester'],
    install_requires=['pygameextra>=2.0.0b45', 'Pillow'],
    package_data={
        'pygameextra_cool_buttons_tester': ['IMAGE_A.png', 'IMAGE_B.png'],
    },
    keywords=['python'],
    entry_points={
        'console_scripts': [
            'pygameextra-cb-tester = pygameextra_cool_buttons_tester.__init__:run',
        ],
    }
)
